/*
 * loadsig.c: retrieval of HSPICE data into MATLAB
 *   written by Michael Perrott while at Silicon
 *   Laboratories.  This code leveraged the work of
 *   Stephen G. Tell (i.e., his Gwave viewer) to
 *   develop the Hspice loading functions.  At this
 *   point, the code here has little similarity to the
 *   routines he provided, but many thanks to Stephen
 *   for providing enough info to get started.
 *   For those wishing to modify this code - good luck!
 *   Unfortunatley, it is quite "hacked" due
 *   to the fact that the actual binary format of Hspice output
 *   was never provided (i.e., I simply updated the
 *   code each time a new issue was found), and due to the
 *   fact that I had little time to develop it.  However,
 *   after years of use, it's pretty solid now.
 *
 *   To compile this code into a mex file for Matlab, simply
 *   run Matlab in the directory that loadsig.c is contained
 *   and then type:
 *   mex loadsig.c
 *   You'll then have a loadsig mex function for whatever
 *   computer (i.e., Sun, Linux, Windows) that you're running
 *   on at the time.
 *
 *   I do ask for one thing for those that use this code - please
 *   keep my name and Silicon Labs attached to it when the user
 *   first executes it (as currently done).  I am not particularly
 *   interested in getting anything for this package other then
 *   recognition, but I do want that since this was a nontrivial
 *   amount of work.  Of course, if you want to throw money at me,
 *   I'm happy to accomodate you. :)  - Michael Perrott 6/3/03
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the MxFree
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

pub mod hspice;
pub mod ngspice;
pub mod shared;

use byteorder::ReadBytesExt;
use std::io::Seek;
use thiserror::Error as TeError;

#[derive(Debug, TeError)]
pub enum Error {
    #[error("{0}")]
    HSpice(#[from] hspice::Error),
    #[error("{0}")]
    NgSpice(#[from] ngspice::Error),
    #[error("given file is not legal signal file")]
    InvalidFmt,
}
type Result<T> = std::result::Result<T, Error>;

pub enum SpiceData {
    HSpice(hspice::HSpiceData),
    NgSpice(ngspice::NgSpiceData),
}

pub fn load_signal<T>(reader: &mut T) -> Result<SpiceData>
where
    T: ReadBytesExt + Seek,
{
    if ngspice::guess_signal(reader) {
        Ok(SpiceData::NgSpice(ngspice::load_signal(reader)?))
    } else if hspice::guess_signal(reader) {
        Ok(SpiceData::HSpice(hspice::load_signal(reader)?))
    } else {
        Err(Error::InvalidFmt)
    }
}
