use byteorder::{BigEndian, LittleEndian, NativeEndian, ReadBytesExt};
use std::io::Result;

#[derive(Debug, Clone, Copy, PartialEq, Eq)]
pub enum Endian {
    NativeEndian,
    BigEndian,
    LittleEndian,
}

pub fn read_int_values<T>(endian: Endian, buf: &mut [u32], reader: &mut T) -> Result<()>
where
    T: ReadBytesExt,
{
    match endian {
        Endian::NativeEndian => reader.read_u32_into::<NativeEndian>(buf)?,
        Endian::BigEndian => reader.read_u32_into::<BigEndian>(buf)?,
        Endian::LittleEndian => reader.read_u32_into::<LittleEndian>(buf)?,
    }
    Ok(())
}

pub fn read_int_value<T>(endian: Endian, reader: &mut T) -> Result<u32>
where
    T: ReadBytesExt,
{
    Ok(match endian {
        Endian::NativeEndian => reader.read_u32::<NativeEndian>()?,
        Endian::BigEndian => reader.read_u32::<BigEndian>()?,
        Endian::LittleEndian => reader.read_u32::<LittleEndian>()?,
    })
}

pub fn read_float_values<T>(endian: Endian, buf: &mut [f32], reader: &mut T) -> Result<()>
where
    T: ReadBytesExt,
{
    match endian {
        Endian::NativeEndian => reader.read_f32_into::<NativeEndian>(buf)?,
        Endian::BigEndian => reader.read_f32_into::<BigEndian>(buf)?,
        Endian::LittleEndian => reader.read_f32_into::<LittleEndian>(buf)?,
    }
    Ok(())
}

pub fn read_float_value<T>(endian: Endian, reader: &mut T) -> Result<f32>
where
    T: ReadBytesExt,
{
    Ok(match endian {
        Endian::NativeEndian => reader.read_f32::<NativeEndian>()?,
        Endian::BigEndian => reader.read_f32::<BigEndian>()?,
        Endian::LittleEndian => reader.read_f32::<LittleEndian>()?,
    })
}

pub fn read_double_values<T>(endian: Endian, buf: &mut [f64], reader: &mut T) -> Result<()>
where
    T: ReadBytesExt,
{
    match endian {
        Endian::NativeEndian => reader.read_f64_into::<NativeEndian>(buf)?,
        Endian::BigEndian => reader.read_f64_into::<BigEndian>(buf)?,
        Endian::LittleEndian => reader.read_f64_into::<LittleEndian>(buf)?,
    }
    Ok(())
}

pub fn read_double_value<T>(endian: Endian, reader: &mut T) -> Result<f64>
where
    T: ReadBytesExt,
{
    Ok(match endian {
        Endian::NativeEndian => reader.read_f64::<NativeEndian>()?,
        Endian::BigEndian => reader.read_f64::<BigEndian>()?,
        Endian::LittleEndian => reader.read_f64::<LittleEndian>()?,
    })
}
